import { IdResponse } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import {
  EmailAlreadyExistsException,
  PhoneAlreadyExistsException,
} from '@modules/user/core/domain/user.errors';
import { UpdateUserRequestDto } from '@modules/user/interface/grpc-controllers/dtos/update-user.request.dto';
import { UpdateUserGrpcController } from '@modules/user/interface/grpc-controllers/update-user.grpc.controller';

import { CommandBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const updateFirstNameUserRequest: UpdateUserRequestDto = {
  id: 'c97b1783-76cf-4840-b298-b90b13c58894',
  firstName: 'Johnny',
};

const updateEmailUserRequest: UpdateUserRequestDto = {
  id: 'c97b1783-76cf-4840-b298-b90b13c58894',
  email: 'john.doe@already.exists.email.com',
};

const updatePhoneUserRequest: UpdateUserRequestDto = {
  id: 'c97b1783-76cf-4840-b298-b90b13c58894',
  phone: '+33611223344',
};

const mockCommandBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => 'c97b1783-76cf-4840-b298-b90b13c58894')
    .mockImplementationOnce(() => {
      throw new EmailAlreadyExistsException();
    })
    .mockImplementationOnce(() => {
      throw new PhoneAlreadyExistsException();
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('Update User Grpc Controller', () => {
  let updateUserGrpcController: UpdateUserGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CommandBus,
          useValue: mockCommandBus,
        },
        UpdateUserGrpcController,
      ],
    }).compile();

    updateUserGrpcController = module.get<UpdateUserGrpcController>(
      UpdateUserGrpcController,
    );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(updateUserGrpcController).toBeDefined();
  });

  it('should update a user', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    const result: IdResponse = await updateUserGrpcController.updateUser(
      updateFirstNameUserRequest,
    );
    expect(result).toBeInstanceOf(IdResponse);
    expect(result.id).toBe('c97b1783-76cf-4840-b298-b90b13c58894');
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if email already exists', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await updateUserGrpcController.updateUser(updateEmailUserRequest);
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.ALREADY_EXISTS);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if phone already exists', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await updateUserGrpcController.updateUser(updatePhoneUserRequest);
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.ALREADY_EXISTS);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await updateUserGrpcController.updateUser(updateFirstNameUserRequest);
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });
});
